package ProjectElectricLocomotive.src;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

public class DrawningElectricLocomotive {

    public EntityElectricLocomotive EntityElectricLocomotive;

    private Double pictureWidth;
    private Double pictureHeight;
    private Double startPosX;
    private Double startPosY;



    public AdditionalClass _additionalClass;

    private final int drawningElectricLocomotiveWidth = 83;
    private final int drawningElectricLocomotiveHeight = 45;

    public void init(int speed, double weight, Color bodyColor, Color additionalColor, boolean electricHorns, boolean batteryPlacement, int wheelsCount) {
        EntityElectricLocomotive = new EntityElectricLocomotive();
        _additionalClass = new AdditionalClass();
        _additionalClass.setAdditonalType(wheelsCount);
        EntityElectricLocomotive.init(speed, weight, bodyColor, additionalColor, electricHorns, batteryPlacement);
        pictureWidth = null;
        pictureHeight = null;
        startPosX = null;
        startPosY = null;
    }

    public boolean setPictureSize(int width, int height) {
        if (drawningElectricLocomotiveHeight > height || drawningElectricLocomotiveWidth > width) {
            return false;
        }
        pictureWidth = (double)width;
        pictureHeight = (double)height;
        if (startPosY != null && startPosY + drawningElectricLocomotiveHeight < pictureHeight) {
            startPosY = pictureHeight - drawningElectricLocomotiveHeight;
        }
        if (startPosX != null && startPosX + drawningElectricLocomotiveWidth < pictureWidth) {
            startPosX = pictureWidth - drawningElectricLocomotiveWidth;
        }
        return true;
    }

    public void setPosition(int x, int y) {
        if (pictureHeight == null || pictureWidth == null) {
            return;
        }

        startPosX = (double)x;
        startPosY = (double)y;

        if (drawningElectricLocomotiveHeight + y > pictureHeight) {
            startPosY = pictureHeight - drawningElectricLocomotiveHeight;
        }
        if (drawningElectricLocomotiveWidth + x > pictureWidth) {
            startPosX = pictureWidth - drawningElectricLocomotiveWidth;
        }
        if (x < 0) {
            startPosX = (double)0;
        }
        if (y < 0) {
            startPosY = (double)0;
        }
    }

    public boolean moveTransport(DirectionType direction) {
        if (EntityElectricLocomotive == null || startPosX == null || startPosY == null) {
            return false;
        }

        switch (direction) {
            case Left:
                if (startPosX - EntityElectricLocomotive.getStep() > 0) {
                    startPosX -= EntityElectricLocomotive.getStep();
                }
                return true;
            case Up:
                if (startPosY - EntityElectricLocomotive.getStep() > 0) {
                    startPosY -= EntityElectricLocomotive.getStep();
                }
                return true;
            case Right:
                if (startPosX + drawningElectricLocomotiveWidth + EntityElectricLocomotive.getStep() < pictureWidth) {
                    startPosX += EntityElectricLocomotive.getStep();
                }
                return true;
            case Down:
                if (startPosY + drawningElectricLocomotiveHeight + EntityElectricLocomotive.getStep() < pictureHeight) {
                    startPosY += EntityElectricLocomotive.getStep();
                }
                return true;
            default:
                return false;
        }
    }

    public Image getImage() {
        // Создаем новое изображение с заданными размерами
        BufferedImage image = new BufferedImage(pictureWidth.intValue(), pictureHeight.intValue(), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2d = image.createGraphics();

        // Вызываем метод прорисовки нашего объекта DrawningElectricLocomotive
        drawTransport(g2d);

        _additionalClass.drawingWheels(g2d, startPosX, startPosY, EntityElectricLocomotive.getAdditionalColor());

        // Освобождаем ресурсы графического контекста
        g2d.dispose();

        return image;
    }

    public void drawTransport(Graphics g) {
        if (EntityElectricLocomotive == null || startPosX == null || startPosY == null) {
            return;
        }

        // Создание перьев и кистей для прорисовки электровоза
        Color blackColor = Color.BLACK;
        Color deepSkyBlueColor = new Color(0, 191, 255);
        Color yellowColor = Color.YELLOW;

        Graphics2D g2d = (Graphics2D) g;

        // Инициализация опорных точек для прорисовки корпуса
        int pointStartX = startPosX.intValue() + 75;
        int pointStartY = startPosY.intValue() + 20;

        Point pointStart = new Point(pointStartX, pointStartY);
        Point point1 = new Point(pointStartX, pointStartY - 15);
        Point point2 = new Point(startPosX.intValue() + 10, pointStartY - 15);
        Point pointFinish = new Point(startPosX.intValue() + 5, pointStartY);

        // Инициализация опорных точек для прорисовки первой "юбки"
        Point point3 = new Point(startPosX.intValue() + 7, startPosY.intValue() + 37);
        Point point4 = new Point(startPosX.intValue(), startPosY.intValue() + 43);
        Point point5 = new Point(startPosX.intValue() + 7, startPosY.intValue() + 43);

        // Инициализация опорных точек для прорисовки второй "юбки"
        Point point6 = new Point(startPosX.intValue() + 69, startPosY.intValue() + 37);
        Point point7 = new Point(startPosX.intValue() + 82, startPosY.intValue() + 43);
        Point point8 = new Point(startPosX.intValue() + 69, startPosY.intValue() + 43);

        // Совокупность точек полигона уголка первой "юбки"
        Point[] firstTrianglePoints = { point3, point4, point5 };

        // Совокупность точек полигона уголка второй "юбки"
        Point[] secondTrianglePoints = { point6, point7, point8 };

        // Прорисовка уголков "юбок"
        g.setColor(blackColor);
        g.fillPolygon(new Polygon(new int[]{ point3.x, point4.x, point5.x }, new int[]{ point3.y, point4.y, point5.y }, 3));
        g.fillPolygon(new Polygon(new int[]{ point6.x, point7.x, point8.x }, new int[]{ point6.y, point7.y, point8.y }, 3));

        // Прорисовка корпуса
        g.drawRect(startPosX.intValue() + 5, startPosY.intValue() + 20, 70, 17);
        g.drawLine(point3.x, point3.y, point4.x, point4.y);
        g.drawLine(pointStart.x, pointStart.y, point1.x, point1.y);
        g.drawLine(point1.x, point1.y, point2.x, point2.y);
        g.drawLine(point2.x, point2.y, pointFinish.x, pointFinish.y);

        // Прорисовка окон
        g.setColor(deepSkyBlueColor);
        g.drawRect(startPosX.intValue() + 12, startPosY.intValue() + 9, 8, 8);
        g.drawRect(startPosX.intValue() + 63, startPosY.intValue() + 9, 8, 8);

        // Прорисовка двух передних колёс

        //g.fillOval(startPosX.intValue() + 24, startPosY.intValue() + 37, 10, 10);
        //g.setColor(blackColor);

        //g.drawOval(startPosX.intValue() + 24, startPosY.intValue() + 37, 8, 8);

        // Прорисовка двух задних колёс
        //g.setColor(Color.YELLOW);
        //g.fillOval(startPosX.intValue() + 63, startPosY.intValue() + 37, 10, 10);
        //g.fillOval(startPosX.intValue() + 53, startPosY.intValue() + 37, 10, 10);
        g.setColor(blackColor);
        //g.drawOval(startPosX.intValue() + 63, startPosY.intValue() + 37, 8, 8);
        //g.drawOval(startPosX.intValue() + 53, startPosY.intValue() + 37, 8, 8);

        // Прорисовка заднего "шлюза"
        g.fillRect(startPosX.intValue() + 75, startPosY.intValue() + 9, 6, 27);








        if (EntityElectricLocomotive.isElectricHorns()) {
            // Инициализация опорных точек для прорисовки "рогов"
            Point pointHorns1 = new Point(startPosX.intValue() + 13, startPosY.intValue() + 5);
            Point pointHorns2 = new Point(startPosX.intValue() + 16, startPosY.intValue() + 2);
            Point pointHorns3 = new Point(startPosX.intValue() + 11, startPosY.intValue());

            g.setColor(EntityElectricLocomotive.getAdditionalColor());

            // Прорисовка "рогов" электровоза
            g.drawLine(pointHorns1.x, pointHorns1.y, pointHorns2.x, pointHorns2.y);
            g.drawLine(pointHorns2.x, pointHorns2.y, pointHorns3.x, pointHorns3.y);

            g.setColor(blackColor);
        }

        if (EntityElectricLocomotive.isBatteryPlacement()) {
            // Инициализация опорных точек для прорисовки молнии на хранилище батарей
            Point pointLightning1 = new Point(startPosX.intValue() + 39, startPosY.intValue() + 37);
            Point pointLightning2 = new Point(startPosX.intValue() + 37, startPosY.intValue() + 39);
            Point pointLightning3 = new Point(startPosX.intValue() + 39, startPosY.intValue() + 40);
            Point pointLightning4 = new Point(startPosX.intValue() + 37, startPosY.intValue() + 41);

            g.setColor(EntityElectricLocomotive.getAdditionalColor());

            // Прорисовка "хранилища батарей" электровоза

            g.fillRect(startPosX.intValue() + 36, startPosY.intValue() + 37, 8, 4);
            g.setColor(yellowColor);
            g.drawLine(pointLightning1.x, pointLightning1.y, pointLightning2.x, pointLightning2.y);
            g.drawLine(pointLightning2.x, pointLightning2.y, pointLightning3.x, pointLightning3.y);
            g.drawLine(pointLightning3.x, pointLightning3.y, pointLightning4.x, pointLightning4.y);

            g.setColor(blackColor);
        }
    }

}
